# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject, Gio # pylint: disable=E0611
import gpgme
import math
import threading

from lockbox_lib.helpers import get_builder
import MessageBox

import gettext
from gettext import gettext as _
gettext.textdomain('lockbox')

key_parameters = """
<GnupgKeyParms format="internal">
Key-Type: RSA
Key-Length: %(size)d
Name-Real: %(name)s
Name-Comment: %(comment)s
Expire-Date: 0
Passphrase: %(passphrase)s
</GnupgKeyParms>
"""

class KeyListModel(Gtk.ListStore):
    def __init__(self):
        Gtk.ListStore.__init__(self)
        self.set_column_types([ GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_BOOLEAN ])
        self.context = gpgme.Context()
        self.context.keylist_mode = gpgme.KEYLIST_MODE_LOCAL
    
    def build(self, chosen_key = ''):
        i = self.context.keylist()
        self.clear()
        for k in i:
            if not k.can_encrypt: continue
            try:
                sk = self.context.get_key(k.subkeys[0].fpr, True)
                del sk
            except:
                continue
            
            li = self.append()
            self.set_value(li, 0, k.uids[0].name)
            self.set_value(li, 1, k.uids[0].comment)
            self.set_value(li, 2, k.subkeys[0].fpr)
            if k.subkeys[0].fpr == chosen_key: self.set_value(li, 3, True)
            else: self.set_value(li, 3, False)

class GenKeyJob(threading.Thread):
    def __init__(self, context, keydata):
        threading.Thread.__init__(self)
        self.context = context
        self.keydata = keydata
    
    def run(self):
        self.context.genkey(self.keydata, None, None)
        print("Thread finished")
        return

class EncryptionSetupDialog(Gtk.Dialog):
    __gtype_name__ = "EncryptionSetupDialog"
    
    current_key = ''
    selected_key = ''
    chosen_key = ''
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated EncryptionSetupDialog object.
        """
        builder = get_builder('EncryptionSetupDialog')
        new_object = builder.get_object('encryptionsetup_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a EncryptionSetupDialog object with it in order to
        finish initializing the start of the new EncryptionSetupDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        
        self.model = KeyListModel()
        self.ui.keyListView.set_model(self.model)
        
        self.settings = Gio.Settings("net.launchpad.lockbox")
        self.current_key = self.settings.get_string('key-fingerprint')
        self.chosen_key = self.current_key
        
        self.refresh_list(None)

    def on_btn_ok_clicked(self, widget):
        self.settings.set_string('key-fingerprint', self.chosen_key)
    
    def add_key(self, widget):
        msg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.OTHER,
                            Gtk.ButtonsType.OK_CANCEL, "Enter key data")
        msg.set_title(_("Create new key"))
        msg.set_markup(_("<big><b>Enter key data</b></big>"))
    
        grid = Gtk.Grid()
        grid.set_row_spacing(4)
        grid.set_column_spacing(4)
    
        label = Gtk.Label(_('Name:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 0, 1, 1)
        nameEntry = Gtk.Entry()
        nameEntry.set_text(_('Lockbox key'))
        grid.attach(nameEntry, 1, 0, 3, 1)
    
        label = Gtk.Label(_('Comment:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 1, 1, 1)
        commentEntry = Gtk.Entry()
        commentEntry.set_text(_('Lockbox disk encryption key'))
        grid.attach(commentEntry, 1, 1, 3, 1)
    
        label = Gtk.Label(_('Key size:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 2, 1, 1)
        sizeSpinButton = Gtk.SpinButton.new_with_range(1024, 4096, 512)
        sizeSpinButton.set_digits(0)
        sizeSpinButton.set_numeric(True)
        sizeSpinButton.set_value(2048)
        grid.attach(sizeSpinButton, 3, 2, 1, 1)
    
        label = Gtk.Label(_('Passphrase:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 3, 1, 1)
        passphraseEntry = Gtk.Entry()
        passphraseEntry.set_visibility(False)
        grid.attach(passphraseEntry, 1, 3, 3, 1)
    
        label = Gtk.Label(_('Repeat passphrase:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 4, 1, 1)
        passphrase2Entry = Gtk.Entry()
        passphrase2Entry.set_visibility(False)
        grid.attach(passphrase2Entry, 1, 4, 3, 1)

        msg.get_message_area().pack_end(grid, False, True, 0)
        grid.show_all()
    
        result = msg.run()
        name = nameEntry.get_text()
        comment = commentEntry.get_text()
        keysize = math.floor(sizeSpinButton.get_value())
        passphrase = passphraseEntry.get_text()
        passphrase2 = passphrase2Entry.get_text()
        msg.destroy()
        del msg
    
        if result != Gtk.ResponseType.OK: return
        if name == '' or passphrase == '' or len(name) < 5:
            MessageBox.error_msg(self, _("Invalid key data"), _("Name and passphrase are required; name must be at least 5 characters long."))
            return

        if passphrase != passphrase2:
            MessageBox.error_msg(self, _("Invalid key data"), _("Passphrase not correctly repeated"))
            return
        
        op_thread = GenKeyJob(self.model.context, key_parameters % { 'name' : name, 'comment' : comment, 'size' : keysize, 'passphrase' : passphrase})

        try:
            MessageBox.process_activity_dialog(self, _("Generating key..."), _("Please perfom some other action\
 (type on the keyboard, move the mouse, utilize the disks) during the key generation."), op_thread)
        except:
            self.process_activity_dialog.destroy()
            MessageBox.error_msg(self, _("Cannot generate key"), _("Crypto backend error"))
            del self.process_activity_dialog
            return
        
        del self.process_activity_dialog
        
        self.refresh_list(None)
    
    def remove_key(self, widget):
        fpr = self.selected_key
        
        if self.selected_key == self.current_key:
            MessageBox.error_msg(self, _("Cannot delete key"), _("This key is currently used by Lockbox"))
            return

        result = MessageBox.warning_question(self, _("Do you really want to delete this key?"), None)

        if result != Gtk.ResponseType.YES: return

        try:
            self.model.context.delete(self.model.context.get_key(fpr, False), True)
        except:
            MessageBox.error_msg(self, _("Cannot delete key"), _("Crypto backend error"))

        self.refresh_list(None)

    def refresh_list(self, widget):
        self.model.build(self.chosen_key)
        
        chosen_key_found = False
        if self.chosen_key != '':
            try:
                li = self.model.get_iter('0')
            except:
                return

            while li:
                if self.model.get_value(li, 2) == self.chosen_key:
                    chosen_key_found = True
                    break
                li = self.model.iter_next(li)
        
        if not chosen_key_found:
            if self.chosen_key == self.current_key:
                self.chosen_key = ''
                self.current_key = ''
            else:
                self.chosen_key = self.current_key

            self.model.build(self.chosen_key)
                
    def change_current_key(self, widget, path):
        self.chosen_key = self.model.get_value(self.model.get_iter(path), 2)
        self.refresh_list(None)
    
    def selection_changed(self, selection):
        (model, li) = selection.get_selected()
        self.ui.removeKeyButton.set_sensitive(li != None)
        if li != None:
            self.selected_key = model.get_value(li, 2)

if __name__ == "__main__":
    dialog = EncryptionSetupDialog()
    dialog.show()
    Gtk.main()

